Domino
======

Dependencies
------------

- [REQUIRED] [Configuration Split](https://www.drupal.org/project/config_split)
- [OPTIONAL] [Features](https://www.drupal.org/project/features)

Configuration
-------------

The module does not come with admin UI for configuration.

We recommend to set `test_users_check_frequency` setting to a value a bit less than cron frequency (domino.settings.yml).
Default value for `test_users_check_frequency` is 1 hour (3600 seconds):

~~~
$config['domino.settings']['test_users_check_frequency'] = '3600';
~~~

Features
--------

* [Default user roles](#user-roles)
* [Default configuration split](#configuration-split)
* [Tests users management for development and production environments](#test-users)
* [Display emails as Drupal messages](#display-emails-as-drupal-messages)

#### User roles

Domino module installs  2 user roles: Developer and Manager.

- Developer role is assumed for someone with full access to the whole system (usually developers)
- Manager role is assumed for people with full access to the content & users (top privileges for non-developer)

Note that currently the Manager role has to be configured manually, it's just done to standardize
commonly used naming of the roles across projects.

#### Configuration split

Domino module includes configuration for 3 configuration splits:

* Development
* Staging
* Production

For start using splits:

1. In your configuration directory (by default - on the same level as the docroot) create "split" folder with 3 children:

    ~~~
    config
    --split
    ----development
    ----production
    ----staging
    --sync
    ~~~

2. Set path to directories on split configuration page or in `settings*.php` file (recommended):

    ~~~
    $config['config_split.config_split.development']['status'] = TRUE;
    $config['config_split.config_split.staging']['status'] = TRUE;
    $config['config_split.config_split.production']['status'] = TRUE;
    ~~~

### Test users

Domino module can handle creating and enabling/disabling test users for different environments.

Domino will try to create and enable test user for every defined role (except Anonymous) on clearing cache if all these conditions will be passed:

* `application_mode` setting is not `production`. `production` is a default value.
* `test_users_password` setting is not empty. By default, it is empty.

If user included in exception list (`test_users_active_on_production`) and `test_users_password_for_production` setting is not empty, this user's password will be updated to value of `test_users_password_for_production`.

Settings can be set in `settings*.php` file:

~~~
$config['domino.settings']['application_mode'] = 'development';
$config['domino.settings']['test_users_password'] = 'PASSWORD';
// Alternatively value can be obtained from environment variable.
// if (!empty($_ENV['TEST_USERS_PASSWORD'])) {
//   $config['domino.settings']['test_users_password'] = $_ENV['TEST_USERS_PASSWORD'];
// }
~~~

Value of `test_users_password` setting will be used as a password for all test users.

Test user username pattern is: `ROLE.test`. It is possible to use an alias for `ROLE`, it can be configured in `settings*.php` file:

~~~
$config['domino.settings']['test_users_usernames_map'] = [
  'administrator' => 'admin',
];
~~~

Note: `test_users_usernames_map` accepts arrays of user role machine names and needed alias.

It is possible to define a list of users which shouldn't be disabled on production via setting `test_users_active_on_production`:

~~~
$config['domino.settings']['test_users_active_on_production'] = [
  'manager.test',
];
~~~

Note: `test_users_active_on_production` accepts usernames.

On production environment Domino will disable all test users on any page load if any of conditions above won't pass.
If user included in exception list and `test_users_password_for_production` setting is not empty, this user won't be disabled, but their password will be updated to value of `test_users_password_for_production`.

### Display emails as Drupal messages

This feature is particularly useful for writing automated tests.
When it's activated, when there's an email sent to any test user in the system, instead of
sending an email, its content is being displayed as Drupal message, so that the content can be verified
as well as links can be clicked / obtained for further testing.

To enable this feature, turn on this setting:

~~~
$config['domino.settings']['display_emails_as_messages'] = TRUE;
~~~

Note that this does not work when `application_mode` is set to `production`.
